/**
 * Text range module for Rangy.
 * Text-based manipulation and searching of ranges and selections.
 *
 * Features
 *
 * - Ability to move range boundaries by character or word offsets
 * - Customizable word tokenizer
 * - Ignores text nodes inside <script> or <style> elements or those hidden by CSS display and visibility properties
 * - Range findText method to search for text or regex within the page or within a range. Flags for whole words and case
 *   sensitivity
 * - Selection and range save/restore as text offsets within a node
 * - Methods to return visible text within a range or selection
 * - innerText method for elements
 *
 * References
 *
 * https://www.w3.org/Bugs/Public/show_bug.cgi?id=13145
 * http://aryeh.name/spec/innertext/innertext.html
 * http://dvcs.w3.org/hg/editing/raw-file/tip/editing.html
 *
 * Part of Rangy, a cross-browser JavaScript range and selection library
 * http://code.google.com/p/rangy/
 *
 * Depends on Rangy core.
 *
 * Copyright 2014, Tim Down
 * Licensed under the MIT license.
 * Version: 1.3.0-alpha.20140822.2
 * Build date: 22 August 2014
 */
!function(e,t){"function"==typeof define&&define.amd?define(["./rangy-core"],e):"undefined"!=typeof module&&"object"==typeof exports?module.exports=e(require("rangy")):e(t.rangy)}(function(e){e.createModule("TextRange",["WrappedSelection"],function(e,t){function n(e,t){function n(t,n,r){for(var i=e.slice(t,n),o={isWord:r,chars:i,toString:function(){return i.join("")}},a=0,c=i.length;c>a;++a)i[a].token=o;s.push(o)}for(var r,i,o,a=e.join(""),s=[],c=0;r=t.wordRegex.exec(a);){if(i=r.index,o=i+r[0].length,i>c&&n(c,i,!1),t.includeTrailingSpace)for(;J.test(e[o]);)++o;n(i,o,!0),c=o}return c<e.length&&n(c,e.length,!1),s}function r(e){var t,n;return e?(t=e.language||Q,n={},j(n,at[t]||at[Q]),j(n,e),n):at[Q]}function i(e){return G(e,it)}function o(e){return G(e,ot)}function a(e){var t=G(e,st);return t.characterOptions=i(t.wordOptions),t}function s(e,t){var n=lt(e,"display",t),r=e.tagName.toLowerCase();return"block"==n&&rt&&ht.hasOwnProperty(r)?ht[r]:n}function c(e){for(var t=p(e),n=0,r=t.length;r>n;++n)if(1==t[n].nodeType&&"none"==s(t[n]))return!0;return!1}function u(e){var t;return 3==e.nodeType&&(t=e.parentNode)&&"hidden"==lt(t,"visibility")}function d(e){return e&&(1==e.nodeType&&!/^(inline(-block|-table)?|none)$/.test(s(e))||9==e.nodeType||11==e.nodeType)}function l(e){return q.isCharacterDataNode(e)||!/^(area|base|basefont|br|col|frame|hr|img|input|isindex|link|meta|param)$/i.test(e.nodeName)}function h(e){for(var t=[];e.parentNode;)t.unshift(e.parentNode),e=e.parentNode;return t}function p(e){return h(e).concat([e])}function f(e){for(;e&&!e.nextSibling;)e=e.parentNode;return e?e.nextSibling:null}function g(e,t){return!t&&e.hasChildNodes()?e.firstChild:f(e)}function v(e){var t=e.previousSibling;if(t){for(e=t;e.hasChildNodes();)e=e.lastChild;return e}var n=e.parentNode;return n&&1==n.nodeType?n:null}function S(e){if(!e||3!=e.nodeType)return!1;var t=e.data;if(""===t)return!0;var n=e.parentNode;if(!n||1!=n.nodeType)return!1;var r=lt(e.parentNode,"whiteSpace");return/^[\t\n\r ]+$/.test(t)&&/^(normal|nowrap)$/.test(r)||/^[\t\r ]+$/.test(t)&&"pre-line"==r}function C(e){if(""===e.data)return!0;if(!S(e))return!1;var t=e.parentNode;return t?c(e)?!0:!1:!0}function N(e){var t=e.nodeType;return 7==t||8==t||c(e)||/^(script|style)$/i.test(e.nodeName)||u(e)||C(e)}function m(e,t){var n=e.nodeType;return 7==n||8==n||1==n&&"none"==s(e,t)}function y(){this.store={}}function x(e,t,n){return function(r){var i=this.cache;if(i.hasOwnProperty(e))return pt++,i[e];ft++;var o=t.call(this,n?this[n]:this,r);return i[e]=o,o}}function T(e,t){this.node=e,this.session=t,this.cache=new y,this.positions=new y}function b(e,t){this.offset=t,this.nodeWrapper=e,this.node=e.node,this.session=e.session,this.cache=new y}function P(){return"[Position("+q.inspectNode(this.node)+":"+this.offset+")]"}function w(){return B(),wt=new Rt}function R(){return wt||w()}function B(){wt&&wt.detach(),wt=null}function E(e,n,r,i){function o(){var e=null;return n?(e=s,c||(s=s.previousVisible(),c=!s||r&&s.equals(r))):c||(e=s=s.nextVisible(),c=!s||r&&s.equals(r)),c&&(s=null),e}r&&(n?N(r.node)&&(r=e.previousVisible()):N(r.node)&&(r=r.nextVisible()));var a,s=e,c=!1,u=!1;return{next:function(){if(u)return u=!1,a;for(var e,t;e=o();)if(t=e.getCharacter(i))return a=e,e;return null},rewind:function(){if(!a)throw t.createError("createCharacterIterator: cannot rewind. Only one position can be rewound.");u=!0},dispose:function(){e=r=null}}}function k(e,t,n){function r(e){for(var t,n,r=[],a=e?i:o,s=!1,c=!1;t=a.next();){if(n=t.character,Y.test(n))c&&(c=!1,s=!0);else{if(s){a.rewind();break}c=!0}r.push(t)}return r}var i=E(e,!1,null,t),o=E(e,!0,null,t),a=n.tokenizer,s=r(!0),c=r(!1).reverse(),u=a(c.concat(s),n),d=s.length?u.slice(Bt(u,s[0].token)):[],l=c.length?u.slice(0,Bt(u,c.pop().token)+1):[];return{nextEndToken:function(){for(var e,t;1==d.length&&!(e=d[0]).isWord&&(t=r(!0)).length>0;)d=a(e.chars.concat(t),n);return d.shift()},previousStartToken:function(){for(var e,t;1==l.length&&!(e=l[0]).isWord&&(t=r(!1)).length>0;)l=a(t.reverse().concat(e.chars),n);return l.pop()},dispose:function(){i.dispose(),o.dispose(),d=l=null}}}function O(e,t,n,r,i){var o,a,s,c,u=0,d=e,l=Math.abs(n);if(0!==n){var h=0>n;switch(t){case M:for(a=E(e,h,null,r);(o=a.next())&&l>u;)++u,d=o;s=o,a.dispose();break;case $:for(var p=k(e,r,i),f=h?p.previousStartToken:p.nextEndToken;(c=f())&&l>u;)c.isWord&&(++u,d=h?c.chars[0]:c.chars[c.chars.length-1]);break;default:throw new Error("movePositionBy: unit '"+t+"' not implemented")}h?(d=d.previousVisible(),u=-u):d&&d.isLeadingSpace&&(t==$&&(a=E(e,!1,null,r),s=a.next(),a.dispose()),s&&(d=s.previousVisible()))}return{position:d,unitsMoved:u}}function L(e,t,n,r){var i=e.getRangeBoundaryPosition(t,!0),o=e.getRangeBoundaryPosition(t,!1),a=r?o:i,s=r?i:o;return E(a,!!r,s,n)}function I(e,t,n){for(var r,i=[],o=L(e,t,n);r=o.next();)i.push(r);return o.dispose(),i}function A(t,n,r){var i=e.createRange(t.node);i.setStartAndEnd(t.node,t.offset,n.node,n.offset);var o=!i.expand("word",r);return o}function W(e,t,n,r,i){function o(e,t){var n=g[e].previousVisible(),r=g[t-1],o=!i.wholeWordsOnly||A(n,r,i.wordOptions);return{startPos:n,endPos:r,valid:o}}for(var a,s,c,u,d,l,h=X(i.direction),p=E(e,h,e.session.getRangeBoundaryPosition(r,h),i.characterOptions),f="",g=[],v=null;a=p.next();)if(s=a.character,n||i.caseSensitive||(s=s.toLowerCase()),h?(g.unshift(a),f=s+f):(g.push(a),f+=s),n){if(d=t.exec(f))if(l){if(c=d.index,u=c+d[0].length,!h&&u<f.length||h&&c>0){v=o(c,u);break}}else l=!0}else if(-1!=(c=f.indexOf(t))){v=o(c,c+t.length);break}return l&&(v=o(c,u)),p.dispose(),v}function _(e){return function(){var t=!!wt,n=R(),r=[n].concat(U.toArray(arguments)),i=e.apply(this,r);return t||B(),i}}function D(e,t){return _(function(n,o,a,s){"undefined"==typeof a&&(a=o,o=M),s=G(s,ct);var c=i(s.characterOptions),u=r(s.wordOptions),d=e;t&&(d=a>=0,this.collapse(!d));var l=O(n.getRangeBoundaryPosition(this,d),o,a,c,u),h=l.position;return this[d?"setStart":"setEnd"](h.node,h.offset),l.unitsMoved})}function F(e){return _(function(t,n){n=i(n);for(var r,o=L(t,this,n,!e),a=0;(r=o.next())&&Y.test(r.character);)++a;o.dispose();var s=a>0;return s&&this[e?"moveStart":"moveEnd"]("character",e?a:-a,{characterOptions:n}),s})}function V(e){return _(function(t,n){var r=!1;return this.changeEachRange(function(t){r=t[e](n)||r}),r})}var M="character",$="word",q=e.dom,U=e.util,j=U.extend,G=U.createOptions,H=q.getBody,z=/^[ \t\f\r\n]+$/,K=/^[ \t\f\r]+$/,Y=/^[\t-\r \u0085\u00A0\u1680\u180E\u2000-\u200B\u2028\u2029\u202F\u205F\u3000]+$/,J=/^[\t \u00A0\u1680\u180E\u2000-\u200B\u202F\u205F\u3000]+$/,Q="en",X=e.Selection.isDirectionBackward,Z=!1,et=!1,tt=!1,nt=!0;!function(){var t=document.createElement("div");t.contentEditable="true",t.innerHTML="<p>1 </p><p></p>";var n=H(document),r=t.firstChild,i=e.getSelection();n.appendChild(t),i.collapse(r.lastChild,2),i.setStart(r.firstChild,0),Z=1==(""+i).length,t.innerHTML="1 <br>",i.collapse(t,2),i.setStart(t.firstChild,0),et=1==(""+i).length,t.innerHTML="1 <p>1</p>",i.collapse(t,2),i.setStart(t.firstChild,0),tt=1==(""+i).length,n.removeChild(t),i.removeAllRanges()}();var rt,it={includeBlockContentTrailingSpace:!0,includeSpaceBeforeBr:!0,includeSpaceBeforeBlock:!0,includePreLineTrailingSpace:!0},ot={includeBlockContentTrailingSpace:!nt,includeSpaceBeforeBr:!et,includeSpaceBeforeBlock:!tt,includePreLineTrailingSpace:!0},at={en:{wordRegex:/[a-z0-9]+('[a-z0-9]+)*/gi,includeTrailingSpace:!1,tokenizer:n}},st={caseSensitive:!1,withinRange:null,wholeWordsOnly:!1,wrap:!1,direction:"forward",wordOptions:null,characterOptions:null},ct={wordOptions:null,characterOptions:null},ut={wordOptions:null,characterOptions:null,trim:!1,trimStart:!0,trimEnd:!0},dt={wordOptions:null,characterOptions:null,direction:"forward"},lt=q.getComputedStyleProperty;!function(){var e=document.createElement("table"),t=H(document);t.appendChild(e),rt="block"==lt(e,"display"),t.removeChild(e)}(),e.features.tableCssDisplayBlock=rt;var ht={table:"table",caption:"table-caption",colgroup:"table-column-group",col:"table-column",thead:"table-header-group",tbody:"table-row-group",tfoot:"table-footer-group",tr:"table-row",td:"table-cell",th:"table-cell"};y.prototype={get:function(e){return this.store.hasOwnProperty(e)?this.store[e]:null},set:function(e,t){return this.store[e]=t}};var pt=0,ft=0,gt={getPosition:function(e){var t=this.positions;return t.get(e)||t.set(e,new b(this,e))},toString:function(){return"[NodeWrapper("+q.inspectNode(this.node)+")]"}};T.prototype=gt;var vt="EMPTY",St="NON_SPACE",Ct="UNCOLLAPSIBLE_SPACE",Nt="COLLAPSIBLE_SPACE",mt="TRAILING_SPACE_BEFORE_BLOCK",yt="TRAILING_SPACE_IN_BLOCK",xt="TRAILING_SPACE_BEFORE_BR",Tt="PRE_LINE_TRAILING_SPACE_BEFORE_LINE_BREAK",bt="TRAILING_LINE_BREAK_AFTER_BR";j(gt,{isCharacterDataNode:x("isCharacterDataNode",q.isCharacterDataNode,"node"),getNodeIndex:x("nodeIndex",q.getNodeIndex,"node"),getLength:x("nodeLength",q.getNodeLength,"node"),containsPositions:x("containsPositions",l,"node"),isWhitespace:x("isWhitespace",S,"node"),isCollapsedWhitespace:x("isCollapsedWhitespace",C,"node"),getComputedDisplay:x("computedDisplay",s,"node"),isCollapsed:x("collapsed",N,"node"),isIgnored:x("ignored",m,"node"),next:x("nextPos",g,"node"),previous:x("previous",v,"node"),getTextNodeInfo:x("textNodeInfo",function(e){var t=null,n=!1,r=lt(e.parentNode,"whiteSpace"),i="pre-line"==r;return i?(t=K,n=!0):("normal"==r||"nowrap"==r)&&(t=z,n=!0),{node:e,text:e.data,spaceRegex:t,collapseSpaces:n,preLine:i}},"node"),hasInnerText:x("hasInnerText",function(e,t){for(var n=this.session,r=n.getPosition(e.parentNode,this.getNodeIndex()+1),i=n.getPosition(e,0),o=t?r:i,a=t?i:r;o!==a;){if(o.prepopulateChar(),o.isDefinitelyNonEmpty())return!0;o=t?o.previousVisible():o.nextVisible()}return!1},"node"),isRenderedBlock:x("isRenderedBlock",function(e){for(var t=e.getElementsByTagName("br"),n=0,r=t.length;r>n;++n)if(!N(t[n]))return!0;return this.hasInnerText()},"node"),getTrailingSpace:x("trailingSpace",function(e){if("br"==e.tagName.toLowerCase())return"";switch(this.getComputedDisplay()){case"inline":for(var t=e.lastChild;t;){if(!m(t))return 1==t.nodeType?this.session.getNodeWrapper(t).getTrailingSpace():"";t=t.previousSibling}break;case"inline-block":case"inline-table":case"none":case"table-column":case"table-column-group":break;case"table-cell":return"	";default:return this.isRenderedBlock(!0)?"\n":""}return""},"node"),getLeadingSpace:x("leadingSpace",function(){switch(this.getComputedDisplay()){case"inline":case"inline-block":case"inline-table":case"none":case"table-column":case"table-column-group":case"table-cell":break;default:return this.isRenderedBlock(!1)?"\n":""}return""},"node")});var Pt={character:"",characterType:vt,isBr:!1,prepopulateChar:function(){var e=this;if(!e.prepopulatedChar){var t=e.node,n=e.offset,r="",i=vt,o=!1;if(n>0)if(3==t.nodeType){var a=t.data,s=a.charAt(n-1),c=e.nodeWrapper.getTextNodeInfo(),u=c.spaceRegex;c.collapseSpaces?u.test(s)?n>1&&u.test(a.charAt(n-2))||(c.preLine&&"\n"===a.charAt(n)?(r=" ",i=Tt):(r=" ",i=Nt)):(r=s,i=St,o=!0):(r=s,i=Ct,o=!0)}else{var d=t.childNodes[n-1];if(d&&1==d.nodeType&&!N(d)&&("br"==d.tagName.toLowerCase()?(r="\n",e.isBr=!0,i=Nt,o=!1):e.checkForTrailingSpace=!0),!r){var l=t.childNodes[n];l&&1==l.nodeType&&!N(l)&&(e.checkForLeadingSpace=!0)}}e.prepopulatedChar=!0,e.character=r,e.characterType=i,e.isCharInvariant=o}},isDefinitelyNonEmpty:function(){var e=this.characterType;return e==St||e==Ct},resolveLeadingAndTrailingSpaces:function(){if(this.prepopulatedChar||this.prepopulateChar(),this.checkForTrailingSpace){var e=this.session.getNodeWrapper(this.node.childNodes[this.offset-1]).getTrailingSpace();e&&(this.isTrailingSpace=!0,this.character=e,this.characterType=Nt),this.checkForTrailingSpace=!1}if(this.checkForLeadingSpace){var t=this.session.getNodeWrapper(this.node.childNodes[this.offset]).getLeadingSpace();t&&(this.isLeadingSpace=!0,this.character=t,this.characterType=Nt),this.checkForLeadingSpace=!1}},getPrecedingUncollapsedPosition:function(e){for(var t,n=this;n=n.previousVisible();)if(t=n.getCharacter(e),""!==t)return n;return null},getCharacter:function(e){function t(){return c||(o=u.getPrecedingUncollapsedPosition(e),c=!0),o}if(this.resolveLeadingAndTrailingSpaces(),this.isCharInvariant)return this.character;var n=["character",e.includeSpaceBeforeBr,e.includeBlockContentTrailingSpace,e.includePreLineTrailingSpace].join("_"),r=this.cache.get(n);if(null!==r)return r;var i,o,a="",s=this.characterType==Nt,c=!1,u=this;return s?(" "!=this.character||t()&&!o.isTrailingSpace&&"\n"!=o.character)&&("\n"==this.character&&this.isLeadingSpace?t()&&"\n"!=o.character&&(a="\n"):(i=this.nextUncollapsed(),i&&(i.isBr?this.type=xt:i.isTrailingSpace&&"\n"==i.character?this.type=yt:i.isLeadingSpace&&"\n"==i.character&&(this.type=mt),"\n"===i.character?(this.type!=xt||e.includeSpaceBeforeBr)&&(this.type!=mt||e.includeSpaceBeforeBlock)&&(this.type==yt&&i.isTrailingSpace&&!e.includeBlockContentTrailingSpace||(this.type!=Tt||i.type!=St||e.includePreLineTrailingSpace)&&("\n"===this.character?i.isTrailingSpace?this.isTrailingSpace||this.isBr&&(i.type=bt,t()&&o.isLeadingSpace&&"\n"==o.character&&(i.character="")):a="\n":" "===this.character&&(a=" "))):a=this.character))):"\n"===this.character&&(!(i=this.nextUncollapsed())||i.isTrailingSpace),this.cache.set(n,a),a},equals:function(e){return!!e&&this.node===e.node&&this.offset===e.offset},inspect:P,toString:function(){return this.character}};b.prototype=Pt,j(Pt,{next:x("nextPos",function(e){var t=e.nodeWrapper,n=e.node,r=e.offset,i=t.session;if(!n)return null;var o,a,s;return r==t.getLength()?(o=n.parentNode,a=o?t.getNodeIndex()+1:0):t.isCharacterDataNode()?(o=n,a=r+1):(s=n.childNodes[r],i.getNodeWrapper(s).containsPositions()?(o=s,a=0):(o=n,a=r+1)),o?i.getPosition(o,a):null}),previous:x("previous",function(e){var t,n,r,i=e.nodeWrapper,o=e.node,a=e.offset,s=i.session;return 0==a?(t=o.parentNode,n=t?i.getNodeIndex():0):i.isCharacterDataNode()?(t=o,n=a-1):(r=o.childNodes[a-1],s.getNodeWrapper(r).containsPositions()?(t=r,n=q.getNodeLength(r)):(t=o,n=a-1)),t?s.getPosition(t,n):null}),nextVisible:x("nextVisible",function(e){var t=e.next();if(!t)return null;var n=t.nodeWrapper,r=t.node,i=t;return n.isCollapsed()&&(i=n.session.getPosition(r.parentNode,n.getNodeIndex()+1)),i}),nextUncollapsed:x("nextUncollapsed",function(e){for(var t=e;t=t.nextVisible();)if(t.resolveLeadingAndTrailingSpaces(),""!==t.character)return t;return null}),previousVisible:x("previousVisible",function(e){var t=e.previous();if(!t)return null;var n=t.nodeWrapper,r=t.node,i=t;return n.isCollapsed()&&(i=n.session.getPosition(r.parentNode,n.getNodeIndex())),i})});var wt=null,Rt=function(){function e(e){var t=new y;return{get:function(n){var r=t.get(n[e]);if(r)for(var i,o=0;i=r[o++];)if(i.node===n)return i;return null},set:function(n){var r=n.node[e],i=t.get(r)||t.set(r,[]);i.push(n)}}}function t(){this.initCaches()}var n=U.isHostProperty(document.documentElement,"uniqueID");return t.prototype={initCaches:function(){this.elementCache=n?function(){var e=new y;return{get:function(t){return e.get(t.uniqueID)},set:function(t){e.set(t.node.uniqueID,t)}}}():e("tagName"),this.textNodeCache=e("data"),this.otherNodeCache=e("nodeName")},getNodeWrapper:function(e){var t;switch(e.nodeType){case 1:t=this.elementCache;break;case 3:t=this.textNodeCache;break;default:t=this.otherNodeCache}var n=t.get(e);return n||(n=new T(e,this),t.set(n)),n},getPosition:function(e,t){return this.getNodeWrapper(e).getPosition(t)},getRangeBoundaryPosition:function(e,t){var n=t?"start":"end";return this.getPosition(e[n+"Container"],e[n+"Offset"])},detach:function(){this.elementCache=this.textNodeCache=this.otherNodeCache=null}},t}();j(q,{nextNode:g,previousNode:v});var Bt=Array.prototype.indexOf?function(e,t){return e.indexOf(t)}:function(e,t){for(var n=0,r=e.length;r>n;++n)if(e[n]===t)return n;return-1};j(e.rangePrototype,{moveStart:D(!0,!1),moveEnd:D(!1,!1),move:D(!0,!0),trimStart:F(!0),trimEnd:F(!1),trim:_(function(e,t){var n=this.trimStart(t),r=this.trimEnd(t);return n||r}),expand:_(function(e,t,n){var o=!1;n=G(n,ut);var a=i(n.characterOptions);if(t||(t=M),t==$){var s,c,u=r(n.wordOptions),d=e.getRangeBoundaryPosition(this,!0),l=e.getRangeBoundaryPosition(this,!1),h=k(d,a,u),p=h.nextEndToken(),f=p.chars[0].previousVisible();if(this.collapsed)s=p;else{var g=k(l,a,u);s=g.previousStartToken()}return c=s.chars[s.chars.length-1],f.equals(d)||(this.setStart(f.node,f.offset),o=!0),c&&!c.equals(l)&&(this.setEnd(c.node,c.offset),o=!0),n.trim&&(n.trimStart&&(o=this.trimStart(a)||o),n.trimEnd&&(o=this.trimEnd(a)||o)),o}return this.moveEnd(M,1,n)}),text:_(function(e,t){return this.collapsed?"":I(e,this,i(t)).join("")}),selectCharacters:_(function(e,t,n,r,i){var o={characterOptions:i};t||(t=H(this.getDocument())),this.selectNodeContents(t),this.collapse(!0),this.moveStart("character",n,o),this.collapse(!0),this.moveEnd("character",r-n,o)}),toCharacterRange:_(function(e,t,n){t||(t=H(this.getDocument()));var r,i,o=t.parentNode,a=q.getNodeIndex(t),s=-1==q.comparePoints(this.startContainer,this.endContainer,o,a),c=this.cloneRange();return s?(c.setStartAndEnd(this.startContainer,this.startOffset,o,a),r=-c.text(n).length):(c.setStartAndEnd(o,a,this.startContainer,this.startOffset),r=c.text(n).length),i=r+this.text(n).length,{start:r,end:i}}),findText:_(function(t,n,i){i=a(i),i.wholeWordsOnly&&(i.wordOptions=r(i.wordOptions),i.wordOptions.includeTrailingSpace=!1);var o=X(i.direction),s=i.withinRange;s||(s=e.createRange(),s.selectNodeContents(this.getDocument()));var c=n,u=!1;"string"==typeof c?i.caseSensitive||(c=c.toLowerCase()):u=!0;var d=t.getRangeBoundaryPosition(this,!o),l=s.comparePoint(d.node,d.offset);-1===l?d=t.getRangeBoundaryPosition(s,!0):1===l&&(d=t.getRangeBoundaryPosition(s,!1));for(var h,p=d,f=!1;;)if(h=W(p,c,u,s,i)){if(h.valid)return this.setStartAndEnd(h.startPos.node,h.startPos.offset,h.endPos.node,h.endPos.offset),!0;p=o?h.startPos:h.endPos}else{if(!i.wrap||f)return!1;s=s.cloneRange(),p=t.getRangeBoundaryPosition(s,!o),s.setBoundary(d.node,d.offset,o),f=!0}}),pasteHtml:function(e){if(this.deleteContents(),e){var t=this.createContextualFragment(e),n=t.lastChild;this.insertNode(t),this.collapseAfter(n)}}}),j(e.selectionPrototype,{expand:_(function(e,t,n){this.changeEachRange(function(e){e.expand(t,n)})}),move:_(function(e,t,n,r){var i=0;if(this.focusNode){this.collapse(this.focusNode,this.focusOffset);var a=this.getRangeAt(0);r||(r={}),r.characterOptions=o(r.characterOptions),i=a.move(t,n,r),this.setSingleRange(a)}return i}),trimStart:V("trimStart"),trimEnd:V("trimEnd"),trim:V("trim"),selectCharacters:_(function(t,n,r,i,o,a){var s=e.createRange(n);s.selectCharacters(n,r,i,a),this.setSingleRange(s,o)}),saveCharacterRanges:_(function(e,t,n){for(var r=this.getAllRanges(),i=r.length,o=[],a=1==i&&this.isBackward(),s=0,c=r.length;c>s;++s)o[s]={characterRange:r[s].toCharacterRange(t,n),backward:a,characterOptions:n};return o}),restoreCharacterRanges:_(function(t,n,r){this.removeAllRanges();for(var i,o,a,s=0,c=r.length;c>s;++s)o=r[s],a=o.characterRange,i=e.createRange(n),i.selectCharacters(n,a.start,a.end,o.characterOptions),this.addRange(i,o.backward)}),text:_(function(e,t){for(var n=[],r=0,i=this.rangeCount;i>r;++r)n[r]=this.getRangeAt(r).text(t);return n.join("")})}),e.innerText=function(t,n){var r=e.createRange(t);r.selectNodeContents(t);var i=r.text(n);return i},e.createWordIterator=function(e,t,n){var o=R();n=G(n,dt);var a=i(n.characterOptions),s=r(n.wordOptions),c=o.getPosition(e,t),u=k(c,a,s),d=X(n.direction);return{next:function(){return d?u.previousStartToken():u.nextEndToken()},dispose:function(){u.dispose(),this.next=function(){}}}},e.noMutation=function(e){var t=R();e(t),B()},e.noMutation.createEntryPointFunction=_,e.textRange={isBlockNode:d,isCollapsedWhitespaceNode:C,createPosition:_(function(e,t,n){return e.getPosition(t,n)})}})},this);